# -*- coding: utf-8 -*-
"""
========================
Setting up the extension
========================

::

    from noseapp.ext.requests import RequestsEx, make_config

    endpoint = make_config()
    endpoint.configure(
        base_url='http://httpbin.org/',
        key='httpbin'
    )
    endpoint.session_configure(
        always_return_json=True,
        raise_on_http_error=True
    )
    requests_ex = RequestsEx(endpoint)

===============
Making requests
===============

Get an API session:

>>> api = requests_ex.get_endpoint_session('httpbin', auth=('user', 'pass'))

Simple *GET* request (if ``always_return_json`` option is set,
response is automatically parsed as JSON into a
``noseapp.datastructures.ModifyDict`` which allows access to values via dot):

>>> api.get('basic-auth/user/pass')
{u'authenticated': True, u'user': u'user'}
>>> r = api.get('basic-auth/user/pass')
>>> r.authenticated
True
>>> r.user
u'user'

*GET* with query-string parameters:

>>> api.get('get', key1='val1')
{u'args': {u'key1': u'val1'},
 <...>
 u'url': u'http://httpbin.org/get?key1=val1'}

*POST* form-encoded data:

>>> api.post('post', key1='val1')
{u'form': {u'key1': u'val1'},
 <...>
 u'url': u'http://httpbin.org/post'}

*POST* JSON data:

>>> api.post('post', {'key1': 'val1'})
{u'data': u'{"key1": "val1"}',
 u'json': {u'key1': u'val1'},
 <...>
 u'url': u'http://httpbin.org/post'}

If ``raise_on_http_error`` option is set,
an exception is raised if response status code is 4xx or 5xx

>>> api.get('status/400')
HTTPError: 400 Client Error: BAD REQUEST

==============
Authentication
==============

You can provide custom authentication class for an endpoint via ``auth_cls``
config parameter:

>>> from requests_oauthlib import OAuth2
>>> endpoint.configure(auth_cls=OAuth2)

Then you can pass authentication parameter in runtime
when getting an API session:

>>> api = requests_ex.get_endpoint_session('oauth2', auth=(client_id, client, token))

"""
from .base import RequestsEx
from .config import make_config

__all__ = (RequestsEx, make_config)



# -*- coding: utf-8 -*-
class Config(dict):
    """
    Extension configuration.
    """
    def configure(self, **params):
        """
        Set endpoint configuration params.

        :param key: A key that will be used to refer to the endpoint.
        :param base_url: Base URL for endpoint.
        :param auth_cls: Authentication class to use with endpoint, \
        e.g. :class:`requests_oauthlib.OAuth2`.
        """
        self.update(params)

    def session_configure(self, **params):
        """
        Set common session params.

        :param bool always_return_json: Parse all responses to JSON if set.
        :param bool raise_on_http_error: Raise HTTPError \
        if response status code >= 400.
        :param: + Any params for :func:`requests.request` or :class:`requests.Session`
        """
        self.setdefault('session_params', {}).update(params)


def make_config():
    """Return :class:`Config` instance with default values."""
    return Config()



# -*- coding: utf-8 -*-
from functools import partial

import requests

from noseapp.core import ExtensionInstaller
from noseapp.datastructures import ModifyDict as JsonObjectHook

from .urlbuilder import BaseUrlBuilder


class RequestsExError(BaseException):
    pass


class Session(object):
    """
    Represents an API/requests session. Actual interface is dependent
    on the ``url_builder`` provided; by default it mimics
    :mod:`requests` interface.

    :param base_url: Base URL for endpoint; if provided, default URL builder \
    will prepend it to all URLs provided in method calls.
    :param url_builder: A custom URL builder callable. \
    See :mod:`noseapp_requests.urlbuilder` documentation for details.
    :param bool always_return_json: Parse all responses to JSON if set.
    :param bool raise_on_http_error: Raise HTTPError \
    if response status code >= 400.
    All other keyword arguments are set directly on :class:`requests.Session`.

    Default interface:

    .. py:method:: get(url, **params)
    .. py:method:: post(url, json_object=None, **params)
    .. py:method:: put(url, json_object=None, **params)
    .. py:method:: delete(url, **params)

    All keyword parameters for ``get`` go into query string,
    for other methods they are urlencoded into request body.
    """
    def __init__(self,
                 base_url=None,
                 url_builder=None,
                 always_return_json=False,
                 raise_on_http_error=False,
                 **session_kwargs):

        if not url_builder:
            url_builder = BaseUrlBuilder(base_url)

        self._url_builder = url_builder
        self.always_return_json = always_return_json
        self.raise_on_http_error = raise_on_http_error

        self._session = requests.Session()
        for attr, value in session_kwargs.items():
            setattr(self._session, attr, value)

    def _request(self, **kwargs):
        resp = self._session.request(**kwargs)

        if self.raise_on_http_error:
            resp.raise_for_status()

        if self.always_return_json:
            return resp.json(object_hook=JsonObjectHook)
        else:
            return resp

    def _dispatch(self, method, *args, **kwargs):
        request_params = self._url_builder(method, *args, **kwargs)

        return self._request(**request_params)

    def __getattr__(self, name):
        return partial(self._dispatch, name)


class Endpoint(object):
    """
    Represents an API endpoint.

    :param Config config: endpoint config
    """
    def __init__(self, config):
        self._auth_cls = config.get('auth_cls')

        self._session_config = config.get('session_params', {})
        self._session_config.update(
            base_url=config.get('base_url'),
            url_builder=config.get('url_builder')
        )

    def get_session(self, **kwargs):
        """
        Get API session for the endpoint.

        If ``auth`` argument is provided and ``auth_cls`` parameter
        was present in config, ``auth`` will be passed as arguments
        to ``auth_cls`` instance. All other arguments will be passed
        to :class:`Session` as is.
        """
        if self._auth_cls is not None:
            try:
                kwargs['auth'] = self._auth_cls(*kwargs['auth'])
            except KeyError:
                raise RequestsExError(
                    'Auth class was set up, but no auth args were provided'
                )

        session_kwargs = self._session_config.copy()
        session_kwargs.update(kwargs)

        return Session(**session_kwargs)


class RequestsEx(object):
    """
    Extension initializer.

    Usage:

    >>> requests_ex = RequestsEx(ENDPOINT1_CONFIG, ENDPOINT2_CONFIG, ...)

    Each endpoint config represents an API endpoint (basically, a base URL)
    that you need access to in tests. So, for example, if you need to call
    your app's API at http://yourapp.test/api/v1/ and also get data from
    some external source at http://databank.test:6847/, you'll provide
    two configs to the extension:::

        app_config = make_config()
        app_config.configure(base_url='http://yourapp.test/api/v1/')
        databank_config = make_config()
        databank_config.configure(base_url='http://databank.test:6847/')
        requests_ex = RequestsEx(app_config, databank_config)

    Each config can be provided a ``key`` that will be used to refer to it.
    By default, ``base_url`` is used as key.
    """
    name = 'requests'

    config_key = 'REQUESTS_EX'

    def __init__(self, *configs):
        self._endpoints = {}

        for config in configs:
            if config.get('base_url') is not None:
                if 'key' not in config:
                    config['key'] = config['base_url']

            self._endpoints[config['key']] = Endpoint(config)

    @classmethod
    def install(cls, app):
        configs = app.config.get(cls.config_key, tuple())

        if not isinstance(configs, (list, tuple)):
            configs = (configs, )

        installer = ExtensionInstaller(cls, configs, {})
        app.shared_extension(cls=installer)

        return installer

    def get_endpoint_session(self, endpoint_key, **kwargs):
        """
        Get API session for endpoint ``endpoint_key``. Any additional ``kwargs``
        will be passed to :meth:`Endpoint.get_session`.
        """
        return self._endpoints[endpoint_key].get_session(**kwargs)



# -*- coding: utf-8 -*-
"""Here be dragons."""
import urlparse


class BaseUrlBuilder(object):
    SUPPORTED_METHODS = ('get', 'post', 'put', 'delete')

    def __init__(self, base_url):
        self.base_url = base_url

    def __call__(self, method, url, json_object=None, **params):
        if method not in self.SUPPORTED_METHODS:
            raise AttributeError(
                "Method {} not supported by {}!".format(method, self.__class__)
            )

        if self.base_url:
            url = urlparse.urljoin(self.base_url, url)

        ret_params = {
            'method': method,
            'url': url
        }

        if json_object is not None:
            ret_params['json'] = json_object

        elif method == 'get' and params:
            ret_params['params'] = params

        elif params:
            ret_params['data'] = params

        return ret_params



# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages


__version__ = '0.0.2'


if __name__ == '__main__':
    setup(
        name='noseapp_requests',
        url='https://github.com/brannt/noseapp_requests',
        version=__version__,
        packages=find_packages(exclude=['docs', 'test*']),
        author='Artem Gorokhov',
        author_email='a.gorohov@corp.mail.ru',
        description='requests extension for noseapp lib',
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Testing',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
        ],
        keywords='noseapp requests testing',
        include_package_data=True,
        zip_safe=False,
        platforms='any',
        install_requires=[
            'noseapp>=1.0.9',
            'requests==2.6.0',
        ],
        extras_require={
            'doc': ['Sphinx']
        }
    )
